/* ----------------------------------------------------------------------------    

 Question Server
 GitLab: <https://gitlab.com/YourFriendlyNeighborhoodDealer/question-node-server>

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program. If not, see <https://www.gnu.org/licenses/>.

 ------------------------------------------------------------------------- */

const express = require('express');
const bodyParser = require('body-parser');
const busboy = require('connect-busboy');
const fs = require('fs');
const app = express();
const http = require('http');
const https = require('https');

const logger = require('./logger.js');
const utils = require('./utils.js');
const actions = require('./actions.js');
const stat = require('./stat.js');

const siteUrl = "https://questionmining.tk"; // http(s)//asd.basd
const recivedFiles = "public/recivedfiles";
const motdFile = "public/motd";
const staticFile = "public/data/static";
const dataFile = "public/data.json";
const countFile = "public/data/count";
const manFile = "public/man.html";
const simpOutFile = "public/simplified";
const inputFile = "stats/inputs";
const msgFile = "stats/msgs";
const logFile = "stats/logs";

// https://certbot.eff.org/
const privkeyFile = "/etc/letsencrypt/live/questionmining.tk/privkey.pem";
const fullchainFile = "/etc/letsencrypt/live/questionmining.tk/fullchain.pem";
const chainFile = "/etc/letsencrypt/live/questionmining.tk/chain.pem";

var certsLoaded = false;
if (utils.FileExists(privkeyFile) && utils.FileExists(fullchainFile) && utils.FileExists(chainFile)) {
	const key = fs.readFileSync(privkeyFile, "utf8");
	const cert = fs.readFileSync(fullchainFile, "utf8");
	const ca = fs.readFileSync(chainFile, "utf8");
	var certs = {
		key: key,
		cert: cert,
		ca: ca
	};
	certsLoaded = true;
}

const port = 8080;
const httpsPort = 8443;

var highlights = ["public", "static", "manual", "isgetting", "postfeedback",
	"postquestions"
];

app.set('view engine', 'ejs');
//app.all('*', function(req, res, next) {
//	if(req.secure) {
//		next();
//	} else {
//		res.redirect('https://' + req.hostname + req.url); 
//	}
//});
app.use(function(req, res, next) {
	res.on('finish', function() {
		Log(req, true, res.statusCode);
		if (res.statusCode != 404)
			stat.LogStat(req.url);
	});
	next();
});
app.use(express.static('public'));
app.use(busboy({
	limits: {
		fileSize: 10 * 1024 * 1024
	}
}));
app.use(bodyParser.json());
app.use(bodyParser.urlencoded({
	limit: '5mb',
	extended: true
}));
app.use(bodyParser.json({
	limit: '5mb'
}));

app.get('/', function(req, res) {
	res.render('main', {
		siteurl: siteUrl
	});
	res.end();
});

app.get('/manual', function(req, res) {
	res.write(utils.ReadFile(manFile));
	res.end();
});

app.get('/public', function(req, res) {
	var response = '@' + utils.ReadFile(motdFile);
	response += utils.ReadFile(staticFile);
	res.write(response);
	res.end();
});

app.get('/static', function(req, res) {
	var response = '@' + utils.ReadFile(motdFile);
	response += utils.ReadFile(staticFile);
	res.write(response);
	res.end();
});

app.get('/legacy', function(req, res) {
	// TODO: make this ejs
	var f = utils.ReadFile(dataFile);
	var r = '<html><body bgcolor="#212127"><head><meta charset="UTF-8">';
	r += '<style>body { font: normal 14px Verdana; color: #999999;}</style>';
	var d = actions.LoadJSON(f).toString().split("\n");
	for (var i = 0; i < d.length; i++)
		r += d[i] + "</br>";
	r += '</head></body></html>';

	res.write(r);
	res.end();
});

app.post('/postfeedback', function(req, res) {
	res.redirect('back');
	logger.Log("[UMSG]: " + req.body.message_field, logger.GetColor("bluebg"), true);
	utils.AppendToFile(logger.GetDateString() + ": " + req.body.message_field, msgFile);
});

app.get('/postfeedback', function(req, res) {
	res.render('main', {
		sdata: utils.ReadFile(staticFile)
	});
});

app.post('/isAdding', function(req, res) {
	res.end('OK');
	Log(req);
	actions.ProcessIncomingRequest(req.body.datatoadd);
	utils.WriteBackup();
});

app.get('/sanityCheck', function(req, res) {
	res.end('Hello guys');
});

app.get('/menuClick', function(req, res) {
	res.redirect("/");
	res.end();
});

app.get('/scriptgit', function(req, res) {
	res.redirect("https://gitlab.com/YourFriendlyNeighborhoodDealer/moodle-test-userscript");
	res.end();
});

app.get('/servergit', function(req, res) {
	res.redirect("https://gitlab.com/YourFriendlyNeighborhoodDealer/question-node-server");
	res.end();
});

app.get('*', function(req, res) {
	res.render('404');
	res.status(404);
	// utils.AppendToFile(logger.GetDateString() + ": " + "404 GET", logFile);
});

app.post('*', function(req, res) {
	res.status(404);
	// utils.AppendToFile(logger.GetDateString() + ": " + "404 POST", logFile);
});

app.route('/badtestsender').post(function(req, res, next) {
	var fstream;
	req.pipe(req.busboy);
	req.busboy.on('file', function(fieldname, file, filename) {
		logger.Log("[UPL]: Uploading: " + filename, logger.GetColor("blue"));

		utils.CreatePath(recivedFiles, true);
		fstream = fs.createWriteStream(recivedFiles + "/" + filename + " | " + Date().toString());
		file.pipe(fstream);
		fstream.on('close', function() {
			logger.Log("[UPL]: Upload Finished of " + filename, logger.GetColor("blue"));
			res.render("uploaded");
		});
		fstream.on('error', function(err) {
			console.log("ERROR:" + err);
			res.end("file uploaded");
		});
	});
});

var msg = "[STRT]: ";
stat.Load();

const httpServer = http.createServer(app);
httpServer.listen(port);
msg += "Server listening on port " + port + " (http)";

if (certsLoaded) {
	const httpsServer = https.createServer(certs, app);
	httpsServer.listen(httpsPort);
	msg += ", and " + httpsPort + " (https)...";
} else {
	logger.Log("Cert files does not exists, starting http only!" ,logger.GetColor("redbg"));
}
logger.Log(msg ,logger.GetColor("yellow"));

function Log(req, toFile, sc) {
	try {
		var ip = req.headers['cf-connecting-ip'] || req.connection.remoteAddress;
		var logEntry = "[RSND]: " + ip + ", " + req.headers['user-agent'] +
			" " + req.method + " ";
		if (sc == 404)
			logEntry += sc + " ";
		logEntry += req.url;
		var color = logger.GetColor("green");
		for (var i = 0; i < highlights.length; i++)
			if (req.url.toLowerCase().includes(highlights[i])) {
				color = logger.GetColor("blue");
				break;
			}
		if (req.url.toLowerCase().includes("isadding"))
			color = logger.GetColor("yellow");
		if (!toFile) {
			logger.Log(logEntry, color);
		} else {
			var defLogs = logger.GetDateString() + ": " + logEntry;
			var extraLogs = "\n\t" + JSON.stringify(req.headers) + "\n\t" + JSON.stringify(req
				.body) + "\n";

			utils.AppendToFile(defLogs, logFile);
		}

	} catch (e) {
		logger.Log("Error at logging lol", logger.GetColor("redbg"), true);
	}
}
